package com.gic.download.utils;

import java.io.*;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSON;
import com.gic.download.qo.HeaderQO;
import org.apache.commons.beanutils.BeanMap;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;

public class ExcelUtils {
    /**
     * logger
     */
    private static final Logger logger = LogManager.getLogger(ExcelUtils.class);

    /**
     * 设置列格式
     *
     * @param sheet
     * @param columnWidthList
     */
    public static boolean setSheetStyle(Sheet sheet, List<Integer> columnWidthList) {
        if (sheet == null || columnWidthList == null) {
            return false;
        }
        // 设置所有列的宽度
        for (int indx = 0, iMax = columnWidthList.size(); indx < iMax; indx++) {
            Integer columnWidth = columnWidthList.get(indx);
            if (columnWidth == null) {
                continue;
            }
            sheet.setColumnWidth(indx, columnWidth.intValue() * 256);
        }
        return true;
    }

    /**
     * 设置行信息
     *
     * @param row
     * @param rowObj
     * @return
     */
    public static boolean setRowInfo(Row row, Collection<Object> rowObj) {
        if (row == null || rowObj == null) {
            if (logger.isInfoEnabled()) {
                logger.info("Row:" + row + ",rowObj" + rowObj);
            }
            return false;
        }
        // 填充每一列数据
        int indxColumn = 0;
        for (Object object : rowObj) {
            Cell cell = row.createCell(indxColumn++);
            if (object == null) {
                if (logger.isDebugEnabled()) {
                    logger.debug("Row:" + row + ",Column:" + indxColumn + ",is empty");
                }
                continue;
            }
            String columnValue = object.toString();
            cell.setCellValue(columnValue);
        }
        return true;
    }

    /**
     * 设置行信息
     *
     * @param row
     * @param rowObj
     */
    @SuppressWarnings({ "rawtypes", "unchecked" })
    public static boolean setRowInfo(Row row, Object rowObj) {
        if (row == null || rowObj == null) {
            if (logger.isInfoEnabled()) {
                logger.info("Row:" + row + ",rowObj" + rowObj);
            }
            return false;
        }
        try {
            Collection<Object> rowContent = null;
            if (rowObj instanceof Map) {
                rowContent = ((Map) rowObj).values();
            } else if (rowObj instanceof Collection) {
                rowContent = (Collection) rowObj;
            } else {
                rowContent = (new BeanMap(rowObj)).values();
            }
            if (rowContent == null || rowContent.isEmpty()) {
                if (logger.isDebugEnabled()) {
                    logger.debug("Row:" + row + ",is empty");
                }
                return false;
            }
            return setRowInfo(row, rowContent);
        } catch (Throwable e) {
            logger.info(rowObj + "convertFailed,row:" + row, e);
            return false;
        }

    }

    /**
     * 将数据写入excel
     *
     * @param sheet
     * @param columnWidth
     * @param header
     * @param content
     * @return
     */
    public static boolean setExcelInfo(Sheet sheet, List<Integer> columnWidth, List<String> header, List<?> content) {
        if (sheet == null) {
            logger.info("sheet is null");
            return false;
        }
        // 设置sheet格式
        setSheetStyle(sheet, columnWidth);

        // 设置头信息
        int indexRow = 0;
        Row row = sheet.createRow(indexRow++);
        setRowInfo(row, header);

        // 如果内容为空 则退出
        if (content == null || content.isEmpty()) {
            logger.info("content is null,cannot write excel");
            return true;
        }
        for (Object rowContent : content) {
            row = sheet.createRow(indexRow++);
            setRowInfo(row, rowContent);
        }
        return true;
    }

    @Deprecated
    public static boolean setHeaderTitle(CellStyle cellStyle, Sheet sheet, List<Integer> columnWidth, List<HeaderQO> headerList, List<?> content) {
        if (sheet == null) {
            logger.info("sheet is null");
            return false;
        }
        // 设置sheet格式
        setSheetStyle(sheet, columnWidth);

        Row row1 = sheet.createRow(0);
        Row row2 = sheet.createRow(1);

        for (int i = 0, n = 0, length = headerList.size(); i < length; i ++) {
            Cell cell1 = row1.createCell(n);
            List<String> sonHeader = headerList.get(i).getSonHeadName();
            String value = headerList.get(i).getHeadName();
            cell1.setCellValue(value);
            cell1.setCellStyle(cellStyle);
            //2层标题
            if (CollectionUtils.isEmpty(sonHeader)) {
                //单标题
                Cell cell2 = row2.createCell(n);
                cell2.setCellStyle(cellStyle);
                sheet.addMergedRegion(new CellRangeAddress(0, 1, n, n));
                n++;
                continue;
            }
            //创建第一行大标题
            sheet.addMergedRegion(new CellRangeAddress(0, 0, n, (n + sonHeader.size() - 1)));
            //赋值
            for (int j = 0, sonLength = sonHeader.size(); j < sonLength; j ++) {
                Cell cell2 = row2.createCell(n++);
                cell2.setCellStyle(cellStyle);
                cell2.setCellValue(sonHeader.get(j));
            }
        }

        // 如果内容为空 则退出
        if (content == null || content.isEmpty()) {
            logger.info("content is null,cannot write excel");
            return true;
        }
        int indexRow = 2;
        for (Object rowContent : content) {
            Row row = sheet.createRow(indexRow++);
            setRowInfo(row, rowContent);
        }
        return true;
    }


    public static boolean doubleHeaderTitle(CellStyle cellStyle, Sheet sheet, List<Integer> columnWidth, LinkedHashMap<String, List<String>> headerMap, List<?> content) {
        if (sheet == null) {
            logger.info("sheet is null");
            return false;
        }
        // 设置sheet格式
        setSheetStyle(sheet, columnWidth);

        Row row1 = sheet.createRow(0);
        Row row2 = sheet.createRow(1);

        int n = 0;
        for (Map.Entry<String, List<String>> entry : headerMap.entrySet()) {
            Cell cell1 = row1.createCell(n);
            List<String> sonHeader = entry.getValue();
            String value = entry.getKey();
            cell1.setCellValue(value);
            cell1.setCellStyle(cellStyle);

            //2层标题
            if (CollectionUtils.isEmpty(sonHeader)) {
                //单标题
                Cell cell2 = row2.createCell(n);
                cell2.setCellStyle(cellStyle);
                sheet.addMergedRegion(new CellRangeAddress(0, 1, n, n));
                n++;
                continue;
            }
            if (sonHeader.size() != 1) {
                //一个单元格，合并操作会报错
                //创建第一行大标题
                sheet.addMergedRegion(new CellRangeAddress(0, 0, n, (n + sonHeader.size() - 1)));
            }
            //赋值
            for (int j = 0, sonLength = sonHeader.size(); j < sonLength; j ++) {
                Cell cell2 = row2.createCell(n++);
                cell2.setCellStyle(cellStyle);
                cell2.setCellValue(sonHeader.get(j));
            }
        }

        // 如果内容为空 则退出
        if (content == null || content.isEmpty()) {
            logger.info("content is null,cannot write excel");
            return true;
        }
        int indexRow = 2;
        for (Object rowContent : content) {
            Row row = sheet.createRow(indexRow++);
            setRowInfo(row, rowContent);
        }
        return true;
    }

    /**
     * 導出到excel
     *
     * @param title
     *        sheet Title
     * @param columnWidthList
     *        所有列的寬度,可以不指定
     * @param content
     *        內容, 每一項為一行,每一行內是List代表所有列
     * @return
     */
    public static Workbook setupXls(String title, List<Integer> columnWidthList, List<List<String>> content) {

        Workbook wb = new HSSFWorkbook();
        Sheet sheet = wb.createSheet(title);
        if (columnWidthList != null) {
            // 设置所有列的宽度
            for (int indx = 0, iMax = columnWidthList.size(); indx < iMax; indx++) {
                Integer columnWidth = columnWidthList.get(indx);
                if (columnWidth == null) {
                    continue;
                }
                sheet.setColumnWidth(indx, columnWidth.intValue() * 256);
            }
        }

        if (content == null || content.isEmpty()) {
            if (logger.isInfoEnabled()) {
                logger.info("content is null,cannot write excel,title:" + title);
            }
            return wb;
        }

        // 遍歷一行
        for (int indxRow = 0, iMaxRow = content.size(); indxRow < iMaxRow; indxRow++) {
            Row row = sheet.createRow(indxRow);
            List<String> rowContent = content.get(indxRow);
            if (rowContent == null || rowContent.isEmpty()) {
                if (logger.isDebugEnabled()) {
                    logger.debug("Row:" + indxRow + ",is empty,title:" + title);
                }
                continue;
            }
            // 填充每一列数据
            for (int indxColumn = 0, iMaxColumn = rowContent.size(); indxColumn < iMaxColumn; indxColumn++) {
                Cell cell = row.createCell(indxColumn);
                String columnValue = rowContent.get(indxColumn);
                if (columnValue == null || columnValue.length() == 0) {
                    if (logger.isDebugEnabled()) {
                        logger.debug("Row:" + indxRow + ",Column:" + indxColumn + ",is empty,title:" + title);
                    }
                    continue;
                }
                cell.setCellValue(columnValue);
            }
        }
        return wb;
    }

    /**
     * 加載Excel 默認實現方式
     *
     * @param wb
     * @return
     */
    public static List<List<String>> loadXls(Workbook wb) {
        // 默認 只讀第一個sheet, 且從第二行開始遍歷,默认读取到最大列
        return loadXls(wb, 0, 1, 0);
    }

    /**
     * 加载excel
     *
     * @param wb
     * @param sheetIndex
     *        要加载excel的sheet页的index
     * @param startRowIndex
     *        要加载Row的index
     * @param iMaxColumn
     *        最大读到Cloumn的index
     * @return List<List<>>
     */
    public static List<List<String>> loadXls(Workbook wb, int sheetIndex, int startRowIndex, int iMaxColumn) {
        List<List<String>> resList = new ArrayList<>();
        if (wb == null || sheetIndex < 0 || startRowIndex < 0 || iMaxColumn < 0) {
            logger.error("param error,return empty list,Workbook:" + wb + ",sheetIndex:" + sheetIndex + ",startRowNo:" + startRowIndex + ",iMaxColumn:" + iMaxColumn);
            return resList;
        }
        Sheet sheet = wb.getSheetAt(sheetIndex);
        if (sheet == null) {
            logger.error("sheet is null,return empty list,Workbook:" + wb + ",sheetIndex:" + sheetIndex + ",startRowNo:" + startRowIndex);
            return resList;
        }

        // 從指定行開始遍歷
        for (int indexRow = startRowIndex, iMaxRow = sheet.getLastRowNum(); indexRow <= iMaxRow; indexRow++) {
            Row row = sheet.getRow(indexRow);
            if (row == null) {
                if (logger.isDebugEnabled()) {
                    logger.debug("Row is null,sheetIndex:" + sheetIndex + ",RowNo:" + indexRow);
                }
                continue;
            }
            List<String> rowContent = new ArrayList<String>();
            // 當最大列為0時 讀取最大CellNum
            if (iMaxColumn == 0) {
                iMaxColumn = row.getLastCellNum();
            }
            boolean hasContent = false;
            for (int indexColumn = 0; indexColumn < iMaxColumn; indexColumn++) {
                String cellValue = null;
                Cell cell = row.getCell(indexColumn);
                if (cell == null) {
                    if (logger.isDebugEnabled()) {
                        logger.debug("Cell is null,sheetIndex:" + sheetIndex + ",RowNo:" + indexRow + ",CellNo:" + indexColumn);
                    }
                } else {
                    cellValue = getCellStrValue(cell);
                }
                // 如果 读到的内容不是空 代表这行有数据
                if (cellValue != null && cellValue.length() > 0) {
                    hasContent = true;
                }
                // 不论当前格是否有数据都加入.
                rowContent.add(cellValue);
            }
            // 这一行有内容 则加入
            if (hasContent) {
                resList.add(rowContent);
            }
        }
        return resList;
    }

    public static String getCellStrValue(Cell cell) {
        String res = "";
        try {
            res = cell.getStringCellValue();
        } catch (Exception e) {
            DecimalFormat df = new DecimalFormat("#");
            res = df.format(cell.getNumericCellValue()) + "";
        }
        return res;
    }

    /**
     *
     * @Description: 将集合转换成字符串输出
     * @param coll
     * @return 设定文件
     * @author albert.su suzy@malam.com
     * @date 2014年5月7日 下午12:35:55
     */
    public static String collectionToCsvString(Collection<?> coll) {
        if (CollectionUtils.isEmpty(coll)) {
            return "";
        }
        StringBuilder sb = new StringBuilder();
        Iterator<?> it = coll.iterator();
        while (it.hasNext()) {
            Object object = it.next();
            if (String.valueOf(object).matches("[0-9,\\.]+") || String.valueOf(object).contains(",")) {
                sb.append("\"\t");
                sb.append(object);
                sb.append("\"");
            } else {
                sb.append("\t");
                sb.append(object);
            }
            if (it.hasNext()) {
                sb.append(",");
            }
        }
        return sb.toString();
    }

    // 以下为糯米的代码, 建议少用

    /**
     * 从InputStream读取Excel workbook
     *
     * @param ins
     * @return
     * @throws IOException
     * @throws FileNotFoundException
     */
    public static HSSFWorkbook readWorkbook(InputStream ins) throws IOException, FileNotFoundException {
        ByteArrayOutputStream byteOS = new ByteArrayOutputStream();
        BufferedInputStream bis = new BufferedInputStream(ins);
        byte[] by = new byte[512];
        int t = bis.read(by, 0, by.length);
        while (t > 0) {
            // read 512
            byteOS.write(by, 0, 512);
            t = bis.read(by, 0, by.length);
        }
        byteOS.close();
        InputStream byteIS = new ByteArrayInputStream(byteOS.toByteArray());
        HSSFWorkbook wbDest = new HSSFWorkbook(byteIS);
        return wbDest;
    }

    public static void writeToResponse(HttpServletResponse response, HSSFWorkbook wb, String fileName) throws IOException {
        response.setContentType("application/ms-download");
        response.setCharacterEncoding("gb2312");
        response.setHeader("Content-Disposition", "filename=" + fileName);
        OutputStream out = response.getOutputStream();
        wb.write(out);
        out.flush();
        out.close();
    }

    /**
     * 判断单元格的格式
     *
     * @param cell
     *        单元格
     * @return String 将excel各种单元格的类型转换为String类型
     */
    public static String getCellStringValue(HSSFCell cell) {
        // 转换后单元格的值
        String value = "";
        if (cell != null) {
            switch (cell.getCellType()) {
                case HSSFCell.CELL_TYPE_STRING:
                    value = cell.getRichStringCellValue().getString();
                    break;
                case HSSFCell.CELL_TYPE_NUMERIC:
                    if (HSSFDateUtil.isCellDateFormatted(cell)) {
                        Date date = HSSFDateUtil.getJavaDate(cell.getNumericCellValue());
                        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                        value = sdf.format(date);
                    } else {
                        DecimalFormat formatter = new DecimalFormat("########");
                        value = formatter.format(cell.getNumericCellValue());
                    }
                    break;

                case HSSFCell.CELL_TYPE_FORMULA:
                    cell.setCellType(HSSFCell.CELL_TYPE_NUMERIC);
                    value = String.valueOf(cell.getNumericCellValue());
                    break;

                case HSSFCell.CELL_TYPE_BLANK:
                    break;
                case HSSFCell.CELL_TYPE_BOOLEAN:
                    break;
                case HSSFCell.CELL_TYPE_ERROR:
                    break;
                default:
                    break;
            }
        }
        return value;
    }

    /**
     * 到出excel的Helper类
     *
     * @author Administrator
     *
     */
    public static class ExcelRowBuilder {

        private HSSFRow row;

        private short index = 0;

        public ExcelRowBuilder(HSSFRow row) {
            this.row = row;
        }

        @SuppressWarnings("deprecation")
        public ExcelRowBuilder addCell(String str) {
            HSSFCell cell = row.createCell(index++);
            cell.setCellValue(new HSSFRichTextString(str));
            return this;
        }

        @SuppressWarnings("deprecation")
        public ExcelRowBuilder addCell(long value) {
            HSSFCell cell = row.createCell(index++);
            cell.setCellValue(value);
            return this;
        }

        @SuppressWarnings("deprecation")
        public ExcelRowBuilder addCell(double value) {
            HSSFCell cell = row.createCell(index++);
            cell.setCellValue(value);
            return this;
        }
    }
}
